@@ -357,8 +357,6 @@ avx512_get_delta(__m256i old_header, __m256i new_header)
                                           0xF, 0xF, 0xF, 0xF);
     v_delta = _mm256_permutexvar_epi32(v_swap32a, v_delta);
 
-    v_delta = _mm256_hadd_epi32(v_delta, v_zeros);
-    v_delta = _mm256_shuffle_epi8(v_delta, v_swap16a);
     v_delta = _mm256_hadd_epi32(v_delta, v_zeros);
     v_delta = _mm256_hadd_epi16(v_delta, v_zeros);
 
@@ -452,7 +450,6 @@ action_avx512_ipv4_set_addrs(struct dp_packet_batch *batch,
 
     DP_PACKET_BATCH_FOR_EACH (i, packet, batch) {
         struct ip_header *nh = dp_packet_l3(packet);
-        ovs_be16 old_csum = ~nh->ip_csum;
 
         /* Load the 20 bytes of the IPv4 header. Without options, which is the
          * most common case it's 20 bytes, but can be up to 60 bytes. */
@@ -465,13 +462,20 @@ action_avx512_ipv4_set_addrs(struct dp_packet_batch *batch,
          * (v_pkt_masked). */
         __m256i v_new_hdr = _mm256_or_si256(v_key_shuf, v_pkt_masked);
 
-        /* Update the IP checksum based on updated IP values. */
-        uint16_t delta = avx512_ipv4_hdr_csum_delta(v_packet, v_new_hdr);
-        uint32_t new_csum = old_csum + delta;
-        delta = csum_finish(new_csum);
+        if (dp_packet_hwol_tx_ip_csum(packet)) {
+            dp_packet_ol_reset_ip_csum_good(packet);
+        } else {
+            ovs_be16 old_csum = ~nh->ip_csum;
 
-        /* Insert new checksum. */
-        v_new_hdr = _mm256_insert_epi16(v_new_hdr, delta, 5);
+            /* Update the IP checksum based on updated IP values. */
+            uint16_t delta = avx512_ipv4_hdr_csum_delta(v_packet, v_new_hdr);
+            uint32_t new_csum = old_csum + delta;
+
+            delta = csum_finish(new_csum);
+
+            /* Insert new checksum. */
+            v_new_hdr = _mm256_insert_epi16(v_new_hdr, delta, 5);
+        }
 
         /* If ip_src or ip_dst has been modified, L4 checksum needs to
          * be updated too. */
@@ -482,9 +486,11 @@ action_avx512_ipv4_set_addrs(struct dp_packet_batch *batch,
             size_t l4_size = dp_packet_l4_size(packet);
 
             if (nh->ip_proto == IPPROTO_UDP && l4_size >= UDP_HEADER_LEN) {
-                /* New UDP checksum. */
                 struct udp_header *uh = dp_packet_l4(packet);
-                if (uh->udp_csum) {
+                if (dp_packet_hwol_l4_is_udp(packet)) {
+                    dp_packet_ol_reset_l4_csum_good(packet);
+                } else if (uh->udp_csum) {
+                    /* New UDP checksum. */
                     uint16_t old_udp_checksum = ~uh->udp_csum;
                     uint32_t udp_checksum = old_udp_checksum + delta_checksum;
                     udp_checksum = csum_finish(udp_checksum);
@@ -497,13 +503,17 @@ action_avx512_ipv4_set_addrs(struct dp_packet_batch *batch,
                 }
             } else if (nh->ip_proto == IPPROTO_TCP &&
                        l4_size >= TCP_HEADER_LEN) {
-                /* New TCP checksum. */
-                struct tcp_header *th = dp_packet_l4(packet);
-                uint16_t old_tcp_checksum = ~th->tcp_csum;
-                uint32_t tcp_checksum = old_tcp_checksum + delta_checksum;
-                tcp_checksum = csum_finish(tcp_checksum);
+                if (dp_packet_hwol_l4_is_tcp(packet)) {
+                    dp_packet_ol_reset_l4_csum_good(packet);
+                } else {
+                    /* New TCP checksum. */
+                    struct tcp_header *th = dp_packet_l4(packet);
+                    uint16_t old_tcp_checksum = ~th->tcp_csum;
+                    uint32_t tcp_checksum = old_tcp_checksum + delta_checksum;
+                    tcp_checksum = csum_finish(tcp_checksum);
 
-                th->tcp_csum = tcp_checksum;
+                    th->tcp_csum = tcp_checksum;
+                }
             }
 
             pkt_metadata_init_conn(&packet->md);
@@ -556,9 +566,6 @@ avx512_ipv6_sum_header(__m512i ip6_header)
                                           0xF, 0xF, 0xF, 0xF);
 
     v_delta = _mm256_permutexvar_epi32(v_swap32a, v_delta);
-
-    v_delta = _mm256_hadd_epi32(v_delta, v_zeros);
-    v_delta = _mm256_shuffle_epi8(v_delta, v_swap16a);
     v_delta = _mm256_hadd_epi32(v_delta, v_zeros);
     v_delta = _mm256_hadd_epi16(v_delta, v_zeros);
 
@@ -568,11 +575,22 @@ avx512_ipv6_sum_header(__m512i ip6_header)
 
 static inline uint16_t ALWAYS_INLINE
 __attribute__((__target__("avx512vbmi")))
-avx512_ipv6_addr_csum_delta(__m512i old_header, __m512i new_header)
+avx512_ipv6_addr_csum_delta(__m512i v_packet, __m512i v_new_hdr,
+                            bool rh_present)
 {
-    uint16_t old_delta = avx512_ipv6_sum_header(old_header);
-    uint16_t new_delta = avx512_ipv6_sum_header(new_header);
-    uint32_t csum_delta = ((uint16_t) ~old_delta) + new_delta;
+    __m512i v_new_hdr_for_cksum = v_new_hdr;
+    uint32_t csum_delta;
+    uint16_t old_delta;
+    uint16_t new_delta;
+
+    if (rh_present) {
+        v_new_hdr_for_cksum = _mm512_mask_blend_epi64(0x18, v_new_hdr,
+                                                      v_packet);
+    }
+
+    old_delta = avx512_ipv6_sum_header(v_packet);
+    new_delta = avx512_ipv6_sum_header(v_new_hdr_for_cksum);
+    csum_delta = ((uint16_t) ~old_delta) + new_delta;
 
     return ~csum_finish(csum_delta);
 }
@@ -655,25 +673,19 @@ action_avx512_set_ipv6(struct dp_packet_batch *batch, const struct nlattr *a)
 
         if (do_csum) {
             size_t l4_size = dp_packet_l4_size(packet);
-            __m512i v_new_hdr_for_cksum = v_new_hdr;
             uint16_t delta_checksum;
 
-            /* In case of routing header being present, checksum should not be
-             * updated for the destination address. */
-            if (rh_present) {
-                v_new_hdr_for_cksum = _mm512_mask_blend_epi64(0x18, v_new_hdr,
-                                                              v_packet);
-            }
-
-            delta_checksum = avx512_ipv6_addr_csum_delta(v_packet,
-                                                         v_new_hdr_for_cksum);
-
             if (proto == IPPROTO_UDP && l4_size >= UDP_HEADER_LEN) {
                 struct udp_header *uh = dp_packet_l4(packet);
-
-                if (uh->udp_csum) {
+                if (dp_packet_hwol_l4_is_udp(packet)) {
+                    dp_packet_ol_reset_l4_csum_good(packet);
+                } else if (uh->udp_csum) {
+                    delta_checksum = avx512_ipv6_addr_csum_delta(v_packet,
+                                                                 v_new_hdr,
+                                                                 rh_present);
                     uint16_t old_udp_checksum = ~uh->udp_csum;
-                    uint32_t udp_checksum = old_udp_checksum + delta_checksum;
+                    uint32_t udp_checksum = old_udp_checksum +
+                                            delta_checksum;
 
                     udp_checksum = csum_finish(udp_checksum);
 
@@ -683,15 +695,26 @@ action_avx512_set_ipv6(struct dp_packet_batch *batch, const struct nlattr *a)
 
                     uh->udp_csum = udp_checksum;
                 }
-            } else if (proto == IPPROTO_TCP && l4_size >= TCP_HEADER_LEN) {
-                struct tcp_header *th = dp_packet_l4(packet);
-                uint16_t old_tcp_checksum = ~th->tcp_csum;
-                uint32_t tcp_checksum = old_tcp_checksum + delta_checksum;
 
-                tcp_checksum = csum_finish(tcp_checksum);
-                th->tcp_csum = tcp_checksum;
+            } else if (proto == IPPROTO_TCP && l4_size >= TCP_HEADER_LEN) {
+                if (dp_packet_hwol_l4_is_tcp(packet)) {
+                    dp_packet_ol_reset_l4_csum_good(packet);
+                } else {
+                    delta_checksum = avx512_ipv6_addr_csum_delta(v_packet,
+                                                                 v_new_hdr,
+                                                                 rh_present);
+                    struct tcp_header *th = dp_packet_l4(packet);
+                    uint16_t old_tcp_checksum = ~th->tcp_csum;
+                    uint32_t tcp_checksum = old_tcp_checksum + delta_checksum;
+
+                    tcp_checksum = csum_finish(tcp_checksum);
+                    th->tcp_csum = tcp_checksum;
+                }
             } else if (proto == IPPROTO_ICMPV6 &&
                        l4_size >= sizeof(struct icmp6_header)) {
+                delta_checksum = avx512_ipv6_addr_csum_delta(v_packet,
+                                                             v_new_hdr,
+                                                             rh_present);
                 struct icmp6_header *icmp = dp_packet_l4(packet);
                 uint16_t old_icmp6_checksum = ~icmp->icmp6_cksum;
                 uint32_t icmp6_checksum = old_icmp6_checksum + delta_checksum;
@@ -704,14 +727,6 @@ action_avx512_set_ipv6(struct dp_packet_batch *batch, const struct nlattr *a)
         }
         /* Write back the modified IPv6 addresses. */
         _mm512_mask_storeu_epi64((void *) nh, 0x1F, v_new_hdr);
-
-        /* Scalar method for setting IPv6 tclass field. */
-        if (key->ipv6_tclass) {
-            uint8_t old_tc = ntohl(get_16aligned_be32(&nh->ip6_flow)) >> 20;
-            uint8_t key_tc = key->ipv6_tclass | (old_tc & ~mask->ipv6_tclass);
-
-            packet_set_ipv6_tc(&nh->ip6_flow, key_tc);
-        }
     }
 }
 #endif /* HAVE_AVX512VBMI */
